CodeQL / Analyze (actions) (push) Canceled after 0s
CodeQL / Analyze (python) (push) Canceled after 0s
pre-commit / pre-commit (push) Canceled after 0s
Multi-Arch CI / setup (push) Canceled after 0s
Publish build_manylinux_x86_64 images / publish_build_manylinux_x86_64 (push) Canceled after 0s
Setup (Multi-Arch) / setup (push) Canceled after 0s
Multi-Arch CI / Linux::${{ fromJSON(needs.setup.outputs.linux_build_config || '{}').build_variant_label || 'skip' }} (push) Canceled after 0s
Publish TheRock Docker image / build-and-push-image (push) Canceled after 0s
Publish no_rocm_image_ubi10 image / publish_no_rocm_image_ubi10 (push) Canceled after 0s
Multi-Arch CI / Windows::${{ fromJSON(needs.setup.outputs.windows_build_config || '{}').build_variant_label || 'skip' }} (push) Canceled after 0s
Publish TheRock Docker image / build-and-push-image (push) Canceled after 0s
Multi-Arch CI / Manifest Diff (push) Canceled after 0s
TheRock Manifest Diff Report / Generate Manifest Diff Report (push) Canceled after 0s
Multi-Arch CI / CI Summary (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04 images / publish_no_rocm_image_ubuntu24_04 (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04_ocl_rt images / publish_no_rocm_image_ubuntu24_04_ocl_rt (push) Canceled after 0s
Publish TheRock Docker image / build-and-push-image (push) Canceled after 0s
Publish TheRock Docker image / build-and-push-image (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04 images / publish_no_rocm_image_ubuntu24_04_rocgdb (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04_rocgdb images / publish_no_rocm_image_ubuntu24_04_rocgdb (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04 images / publish_no_rocm_image_ubuntu24_04_ocl_rt (push) Canceled after 0s
Publish TheRock Docker image / build-and-push-image (push) Canceled after 0s
Unit Tests / Unit Tests :: ubuntu-24.04 (push) Canceled after 0s
Unit Tests / Unit Tests :: windows-2022 (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04 images / publish_no_rocm_image_ubuntu24_04_media (push) Canceled after 0s
Unit Tests / Unit Tests Summary (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04 images / publish_no_rocm_image_ubuntu24_04_openmpi (push) Canceled after 0s
113 lines
2.6 KiB
C++
113 lines
2.6 KiB
C++
//===----------------------------------------------------------------------===//
|
|
//
|
|
// Part of the LLVM Project, under the Apache License v2.0 with LLVM Exceptions.
|
|
// See https://llvm.org/LICENSE.txt for license information.
|
|
// SPDX-License-Identifier: Apache-2.0 WITH LLVM-exception
|
|
//
|
|
//===----------------------------------------------------------------------===//
|
|
|
|
#ifndef TEST_SUPPORT_COUNTING_PREDICATES_H
|
|
#define TEST_SUPPORT_COUNTING_PREDICATES_H
|
|
|
|
#include <cuda/std/cstddef>
|
|
#include <cuda/std/utility>
|
|
|
|
#include "test_macros.h"
|
|
|
|
template <typename Predicate, typename Arg>
|
|
struct unary_counting_predicate
|
|
{
|
|
public:
|
|
typedef Arg argument_type;
|
|
typedef bool result_type;
|
|
|
|
__host__ __device__ constexpr unary_counting_predicate(Predicate p)
|
|
: p_(p)
|
|
, count_(0)
|
|
{}
|
|
|
|
__host__ __device__ constexpr bool operator()(const Arg& a)
|
|
{
|
|
++count_;
|
|
return p_(a);
|
|
}
|
|
__host__ __device__ constexpr size_t count() const
|
|
{
|
|
return count_;
|
|
}
|
|
__host__ __device__ constexpr void reset()
|
|
{
|
|
count_ = 0;
|
|
}
|
|
|
|
private:
|
|
Predicate p_;
|
|
size_t count_;
|
|
};
|
|
|
|
template <typename Predicate, typename Arg1, typename Arg2 = Arg1>
|
|
struct binary_counting_predicate
|
|
{
|
|
public:
|
|
typedef Arg1 first_argument_type;
|
|
typedef Arg2 second_argument_type;
|
|
typedef bool result_type;
|
|
|
|
__host__ __device__ constexpr binary_counting_predicate(Predicate p)
|
|
: p_(p)
|
|
, count_(0)
|
|
{}
|
|
|
|
__host__ __device__ constexpr bool operator()(const Arg1& a1, const Arg2& a2)
|
|
{
|
|
++count_;
|
|
return p_(a1, a2);
|
|
}
|
|
__host__ __device__ constexpr size_t count() const
|
|
{
|
|
return count_;
|
|
}
|
|
__host__ __device__ constexpr void reset()
|
|
{
|
|
count_ = 0;
|
|
}
|
|
|
|
private:
|
|
Predicate p_;
|
|
size_t count_;
|
|
};
|
|
|
|
template <class Predicate>
|
|
class counting_predicate
|
|
{
|
|
Predicate pred_;
|
|
int* count_ = nullptr;
|
|
|
|
public:
|
|
constexpr counting_predicate() = default;
|
|
__host__ __device__ constexpr counting_predicate(Predicate pred, int& count)
|
|
: pred_(cuda::std::move(pred))
|
|
, count_(&count)
|
|
{}
|
|
|
|
template <class... Args>
|
|
__host__ __device__ constexpr auto operator()(Args&&... args) -> decltype(pred_(cuda::std::forward<Args>(args)...))
|
|
{
|
|
++(*count_);
|
|
return pred_(cuda::std::forward<Args>(args)...);
|
|
}
|
|
|
|
template <class... Args>
|
|
__host__ __device__ constexpr auto operator()(Args&&... args) const
|
|
-> decltype(pred_(cuda::std::forward<Args>(args)...))
|
|
{
|
|
++(*count_);
|
|
return pred_(cuda::std::forward<Args>(args)...);
|
|
}
|
|
};
|
|
|
|
template <class Predicate>
|
|
counting_predicate(Predicate pred, int& count) -> counting_predicate<Predicate>;
|
|
|
|
#endif // TEST_SUPPORT_COUNTING_PREDICATES_H
|